# -*- coding: utf-8 -*-

import difflib
import copy
import re
from humanize import intword

fields_good_name = {
    "rank": "Rank",
    "symbol": "Symbol",
    "price": "Price (USD)",
    "percent_change_24h": "Change (24H)",
    "percent_change_1h": "Change (1H)",
    "market_cap": "Market Cap (USD)"
}


class Colors:
    YELLOW = '\033[93m'
    RED = '\033[91m'
    GREEN = '\033[92m'
    BLUE = '\033[94m'
    ENDLINE = '\033[0m'

    @staticmethod
    def color_data(data):
        data[0][0] = Colors.YELLOW + data[0][0]
        data[0][len(data[0]) - 1] = data[0][len(data[0]) - 1] + Colors.ENDLINE

        for item in data[1:]:
            if re.search('-\d+\.\d+', item[3]):
                item[3] = Colors.RED + item[3] + '%' + Colors.ENDLINE
            else:
                item[3] = Colors.GREEN + item[3] + '%' + Colors.ENDLINE
            if re.search('-\d+\.\d+', item[4]):
                item[4] = Colors.RED + item[4] + '%' + Colors.ENDLINE
            else:
                item[4] = Colors.GREEN + item[4] + '%' + Colors.ENDLINE


def process_data(data, fields=['rank', 'symbol', 'price_usd', 'percent_change_24h', 'percent_change_1h', 'market_cap_usd'],
                 currency='USD', humanize=True):

    if currency != 'USD':
        pos = 0
        for field in fields:
            fields[pos] = field.replace('usd', currency.lower())
            pos += 1

    # Initialize structure
    tabulated_data = []
    tabulated_data.append(copy.copy(fields))   # Headers in position 0

    pos = 0
    for header in tabulated_data[0]:   # Headers in position 0
        good_header = difflib.get_close_matches(header, fields_good_name.keys())[0]
        tabulated_data[0][pos] = fields_good_name[good_header]
        if good_header in ['price', 'market_cap']:
            tabulated_data[0][pos] = tabulated_data[0][pos].replace('USD', currency)
        pos += 1

    for item in data:
        tab_item = []
        for field in fields:
            if humanize and re.search('market_cap*', field):
                tab_item.append(intword(int(float(item[field]))))
            else:
                tab_item.append(item[field])
        tabulated_data.append(copy.copy(tab_item))

    return tabulated_data


def find_data(data, symbols):

    symbols = [x.upper() for x in symbols]   # Convert to upper-case
    filtered_items = []
    for item in data:
        if item['symbol'] in symbols:
            filtered_items.append(item)

    return filtered_items



# -*- coding: utf-8 -*-

import argparse
from requests import get
from tabulate import tabulate
from pycoinmon.common import process_data, find_data, Colors
from pycoinmon.ascii import ascii_title, process_title
from pycoinmon.metadata import Metadata
#from terminaltables import AsciiTable
from colorama import init, deinit
import os
import time

class PyCoinmon(object):

    def __init__(self):
        if 'PYCHARM_HOSTED' not in os.environ:  # Exclude PyCharm IDE from colorama init
            init()
        self.meta = Metadata()
        self.sourceURL = "https://api.coinmarketcap.com/v1/ticker"

        # Parse arguments provided
        parser = argparse.ArgumentParser()
        parser.add_argument('-v', '--version', action='version', version=self.meta.get_version())
        parser.add_argument('-c', '--convert', dest='currency', help='Convert to your fiat currency', default='usd',
                            choices=['USD', 'AUD', 'BRL', 'CAD', 'CHF', 'CLP', 'CNY', 'CZK', 'DKK', 'EUR', 'GBP', 'HKD',
                                     'HUF', 'IDR', 'ILS', 'INR', 'JPY', 'KRW', 'MXN', 'MYR', 'NOK', 'NZD', 'PHP', 'PKR',
                                     'PLN', 'RUB', 'SEK', 'SGD', 'THB', 'TRY', 'TWD', 'ZAR'],
                            type=lambda s: s.upper())
        parser.add_argument('-f', '--find', dest='symbol',
                            help='Find specific coin data with coin symbol (can be a space seperated list)',
                            metavar='S', type=str, nargs='+')
        parser.add_argument('-l', '--layout', dest='template', help='Select table layout', default='grid',
                            choices=['plain', 'simple', 'grid', 'fancy_grid', 'pipe', 'orgtbl', 'presto', 'psql',
                                     'rst'],
                            type=lambda s: s.lower())
        parser.add_argument('-t', '--top', dest='index',
                            help='Show the top coins ranked from 1 - index according to the market cap', type=int, default=10)
        parser.add_argument('-u', '--update', dest='frequency',
                            help='Update data with frequency specified in seconds. If 0 just show one time.',
                            type=self.check_positive)
        parser.add_argument('-H', '--humanize', dest='humanize', action='store_true', help='Show market cap as a humanized number')
        self.args = parser.parse_args()

    def __del__(self):
        if deinit is not None:   # Prevent error when argparse raise exception
            deinit()

    @staticmethod
    def check_positive(value):
        ivalue = int(value)
        if ivalue <= 0:
            raise argparse.ArgumentTypeError("%s is an invalid positive int value" % value)
        return ivalue

    def request_values(self):
        payload = {'limit': self.args.index, 'convert': self.args.currency}
        return get(self.sourceURL, params=payload)

    def print_values(self):
        # Update values
        response = self.request_values()

        # Clear screen
        if os.name == 'nt':
            os.system('cls')   # on windows
        else:
            os.system('clear')   # on linux / os x

        # Redraw data
        print(process_title(ascii_title))
        # print(Colors.YELLOW + ascii_title + Colors.ENDLINE)
        if self.args.symbol:
            filtered_data = find_data(response.json(), self.args.symbol)
        else:
            filtered_data = response.json()
        tabulated_data = process_data(filtered_data, currency=self.args.currency, humanize=self.args.humanize)
        Colors.color_data(tabulated_data)
        # table = AsciiTable(tabulated_data)
        # print(table.table)
        print(tabulate(tabulated_data, headers='firstrow', tablefmt=self.args.template))
        print("\n")

    def run(self):
        if self.args.frequency:
            try:
                while True:
                    self.print_values()
                    print("Press Ctrl + C to exit.")
                    time.sleep(int(self.args.frequency))
            except KeyboardInterrupt:
                pass
        else:
            self.print_values()


if __name__ == "__main__":
    pycoinmon = PyCoinmon()
    pycoinmon.run()



from setuptools import setup
from pycoinmon.metadata import Metadata

metadata = Metadata()


def requirements():
    """Build the requirements list for this project"""
    requirements_list = []

    with open('requirements.txt') as requirements:
        for install in requirements:
            requirements_list.append(install.strip())

    return requirements_list

long_description = """"
🐍 Python Port 🐍 Based on `COINMON`_

    💰 Cryptocurrency price ticker CLI.

Check cryptocurrencies’ prices, changes on your console. Best CLI tool
for those who are both **Crypto investors** and **Engineers**.

All data comes from `coinmarketcap.com`_ APIs.

Installation
------------

You can install or upgrade pycoinmon with:

``$ pip install pycoinmon --upgrade``

Or you can install from source with:

::

    $ git clone https://github.com/RDCH106/pycoinmon.git --recursive
    $ cd pycoinmon
    $ pip install .

Usage
-----

To check the top 10 cryptocurrencies ranked by their market cap, simply
execute

::

    $ pycoinmon

Options
~~~~~~~

You can use the ``-c`` (or ``--convert``) with the fiat currency symbol
to find in terms of another currency. The default currency is USD and it
supports AUD, BRL, CAD, CHF, CLP, CNY, CZK, DKK, EUR, GBP, HKD, HUF,
IDR, ILS, INR, JPY, KRW, MXN, MYR, NOK, NZD, PHP, PKR, PLN, RUB, SEK,
SGD, THB, TRY, TWD, ZAR.

::

    $ pycoinmon -c eur // convert prices to Euro
    $ pycoinmon -c jpy // convert prices to Yen

You can use the ``-f`` (or ``--find``) with coin symbol to search
cryptocurrencies. You can add symbols seperated by space.

::

    $ pycoinmon -f btc // search coins included keyword btc
    $ pycoinmon -f btc eth // search coins included keyword btc or eth

You can use the ``-l`` (or ``--layout``) with template name to print the
table with different style. The default layout template is grid and it
supports plain, simple, fancy_grid, pipe, orgtbl, ‘presto’, ‘psql’,
‘rst’.

::

    $ pycoinmon -l plain // show table with plain style
    $ pycoinmon -l fancy_grid // show table with fancy_grid style

You can use the ``-t`` (or ``--top``) with the index to find the top n
cryptocurrencies ranked by their market cap.

::

    $ pycoinmon -t 50 // find top 50
    $ pycoinmon -t 1000 // find top 1000

You can use the ``-u`` (or ``--update``) with the refresh frequency in
seconds. The value must be bigger than 0.

::

    $ pycoinmon -u 10 // update data each 10 seconds

You can use the ``-H`` (or ``--humanize``) to display market cap in
humanized format.

::

    $ pycoinmon -H // show market cap in humanized format like 58.9 billion

You can use the ``-h`` (or ``--help``) to find all valid options of
pycoinmon

::

    $ pycoinmon -h

Screenshot
----------

.. figure:: https://raw.githubusercontent.com/RDCH106/pycoinmon/master/pycoinmon.png
   :alt: pycoinmon screenshot

.. _COINMON: https://github.com/bichenkk/coinmon
.. _coinmarketcap.com: https://coinmarketcap.com/
    """


setup(
    name = 'pycoinmon',
    packages = ['pycoinmon'],
    install_requires = requirements(),
    version = metadata.get_version(),
    license = 'MIT',
    description = 'The cryptocurrency price tool on CLI',
    long_description= long_description,
    author = metadata.get_author(),
    author_email = 'contact@rdch106.hol.es',
    url = 'https://github.com/RDCH106/pycoinmon',
    download_url = 'https://github.com/RDCH106/pycoinmon/archive/v'+metadata.get_version()+'.tar.gz',
    entry_points={
        'console_scripts': ['pycoinmon=pycoinmon.main:main'],
    },
    keywords = 'bitcoin criptocurrency crypto ticker python cli price-tracker command-line',
    classifiers = ['Programming Language :: Python',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3.2',
                   'Programming Language :: Python :: 3.3',
                   'Programming Language :: Python :: 3.4',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6'],
)